@@ -1,22 +1,20 @@
 /* find -- search for files in a directory hierarchy
    Copyright (C) 1990, 91, 92, 93, 94, 2000, 
-                 2003, 2004, 2005 Free Software Foundation, Inc.
+                 2003, 2004, 2005, 2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-   USA.*/
-
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 /* GNU find was written by Eric Decker <cire@cisco.com>,
    with enhancements by David MacKenzie <djm@gnu.org>,
    Jay Plett <jay@silence.princeton.nj.us>,
@@ -37,12 +35,8 @@
 #include <errno.h>
 #include <assert.h>
 
-
-#ifdef HAVE_FCNTL_H
+#include <sys/stat.h>
 #include <fcntl.h>
-#else
-#include <sys/file.h>
-#endif
 #include <openat.h>
 
 #include "xalloc.h"
@@ -50,7 +44,7 @@
 #include "canonicalize.h"
 #include <modetype.h>
 
-#include "closeout.h"
+#include "closein.h"
 #include "savedirinfo.h"
 #include "buildcmd.h"
 #include "dirname.h"
@@ -136,7 +130,7 @@ main (int argc, char **argv)
   program_name = argv[0];
   state.exit_status = 0;
 
-  /* Set the option defaults before we do the the locale
+  /* Set the option defaults before we do the locale
    * initialisation as check_nofollow() needs to be executed in the
    * POSIX locale.
    */
@@ -147,7 +141,7 @@ main (int argc, char **argv)
 #endif
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
-  atexit (close_stdout);
+  atexit (close_stdin);
 
   /* Check for -P, -H or -L options. */
   end_of_leading_options = process_leading_options(argc, argv);
@@ -174,7 +168,7 @@ main (int argc, char **argv)
    * To avoid bailing out when something gets automounted, it checks if 
    * the target directory appears to have had a directory mounted on it as
    * we chdir()ed.  The problem with this is that in order to notice that 
-   * a filesystem was mounted, we would need to lstat() all the mount points.
+   * a file system was mounted, we would need to lstat() all the mount points.
    * That strategy loses if our machine is a client of a dead NFS server.
    *
    * Hence if safely_chdir() and wd_sanity_check() can manage without needing 
@@ -198,7 +192,6 @@ main (int argc, char **argv)
       close (starting_desc);
       starting_desc = -1;
     }
-  assert(starting_desc >= 0);
 
   if (starting_desc < 0)
     {
@@ -275,7 +268,7 @@ specific_dirname(const char *dir)
 
 
 
-/* Return non-zero if FS is the name of a filesystem that is likely to
+/* Return non-zero if FS is the name of a file system that is likely to
  * be automounted
  */
 static int
@@ -294,8 +287,8 @@ static size_t num_mounted_devices = 0u;
 static void
 init_mounted_dev_list(int mandatory)
 {
-  assert(NULL == mounted_devices);
-  assert(0 == num_mounted_devices);
+  assert (NULL == mounted_devices);
+  assert (0 == num_mounted_devices);
   mounted_devices = get_mounted_devices(&num_mounted_devices);
   if (mandatory && (NULL == mounted_devices))
     {
@@ -362,15 +355,15 @@ get_mount_state(dev_t newdev)
 /* We stat()ed a directory, chdir()ed into it (we know this 
  * since direction is TraversingDown), stat()ed it again,
  * and noticed that the device numbers are different.  Check
- * if the filesystem was recently mounted. 
+ * if the file system was recently mounted. 
  * 
  * If it was, it looks like chdir()ing into the directory
- * caused a filesystem to be mounted.  Maybe automount is
+ * caused a file system to be mounted.  Maybe automount is
  * running.  Anyway, that's probably OK - but it happens
  * only when we are moving downward.
  *
  * We also allow for the possibility that a similar thing
- * has happened with the unmounting of a filesystem.  This
+ * has happened with the unmounting of a file system.  This
  * is much rarer, as it relies on an automounter timeout
  * occurring at exactly the wrong moment.
  */
@@ -388,7 +381,7 @@ dirchange_is_fatal(const char *specific_what,
       if (!silent)
 	{
 	  error (0, 0,
-		 _("Warning: filesystem %s has recently been unmounted."),
+		 _("Warning: file system %s has recently been unmounted."),
 		 safely_quote_err_filename(0, specific_what));
 	}
       break;
@@ -398,7 +391,7 @@ dirchange_is_fatal(const char *specific_what,
       if (!silent)
 	{
 	  error (0, 0,
-		 _("Warning: filesystem %s has recently been mounted."),
+		 _("Warning: file system %s has recently been mounted."),
 		 safely_quote_err_filename(0, specific_what));
 	}
       break;
@@ -418,13 +411,13 @@ dirchange_is_fatal(const char *specific_what,
 
 /* Examine the results of the stat() of a directory from before we
  * entered or left it, with the results of stat()ing it afterward.  If
- * these are different, the filesystem tree has been modified while we
+ * these are different, the file system tree has been modified while we
  * were traversing it.  That might be an attempt to use a race
  * condition to persuade find to do something it didn't intend
  * (e.g. an attempt by an ordinary user to exploit the fact that root
- * sometimes runs find on the whole filesystem).  However, this can
+ * sometimes runs find on the whole file system).  However, this can
  * also happen if automount is running (certainly on Solaris).  With 
- * automount, moving into a directory can cause a filesystem to be 
+ * automount, moving into a directory can cause a file system to be 
  * mounted there.
  *
  * To cope sensibly with this, we will raise an error if we see the
@@ -438,11 +431,11 @@ dirchange_is_fatal(const char *specific_what,
  *
  * If the device number and inode are both the same, we are happy.
  *
- * If a filesystem is (un)mounted as we chdir() into the directory, that 
- * may mean that we're now examining a section of the filesystem that might 
+ * If a file system is (un)mounted as we chdir() into the directory, that 
+ * may mean that we're now examining a section of the file system that might 
  * have been excluded from consideration (via -prune or -quit for example).
  * Hence we print a warning message to indicate that the output of find 
- * might be inconsistent due to the change in the filesystem.
+ * might be inconsistent due to the change in the file system.
  */
 static boolean
 wd_sanity_check(const char *thing_to_stat,
@@ -494,7 +487,7 @@ wd_sanity_check(const char *thing_to_stat,
 	  {
 	    fstype = filesystem_type(newinfo, current_dir);
 	    error (1, 0,
-		   _("%s%s changed during execution of %s (old device number %ld, new device number %ld, filesystem type is %s) [ref %ld]"),
+		   _("%s%s changed during execution of %s (old device number %ld, new device number %ld, file system type is %s) [ref %ld]"),
 		   safely_quote_err_filename(0, specific_what),
 		   parent ? "/.." : "",
 		   safely_quote_err_filename(1, progname),
@@ -531,7 +524,7 @@ wd_sanity_check(const char *thing_to_stat,
       
       error ((isfatal == FATAL_IF_SANITY_CHECK_FAILS) ? 1 : 0,
 	     0,			/* no relevant errno value */
-	     _("%s%s changed during execution of %s (old inode number %ld, new inode number %ld, filesystem type is %s) [ref %ld]"),
+	     _("%s%s changed during execution of %s (old inode number %ld, new inode number %ld, file system type is %s) [ref %ld]"),
 	     safely_quote_err_filename(0, specific_what), 
 	     parent ? "/.." : "",
 	     safely_quote_err_filename(1, progname),
@@ -723,7 +716,7 @@ safely_chdir_lstat(const char *dest,
 	      else if (ENOTDIR == saved_errno)
 		{
 		  /* This can happen if the we stat a directory,
-		   * and then filesystem activity changes it into 
+		   * and then file system activity changes it into 
 		   * a non-directory.
 		   */
 		  saved_errno = 0;	/* don't issue err msg */
@@ -779,11 +772,11 @@ safely_chdir_lstat(const char *dest,
     }
   
   *did_stat = statflag;
-  assert(rv_set);
+  assert (rv_set);
   return rv;
 }
 
-#if defined(O_NOFOLLOW)
+#if defined O_NOFOLLOW
 /* Safely change working directory to the specified subdirectory.  If
  * we are not allowed to follow symbolic links, we use open() with
  * O_NOFOLLOW, followed by fchdir().  This ensures that we don't
@@ -953,13 +946,13 @@ at_top (char *pathname,
 		       struct stat *pstat))
 {
   int dirchange;
-  char *parent_dir = dir_name(pathname);
-  char *base = base_name(pathname);
-  
+  char *parent_dir = dir_name (pathname);
+  char *base = last_component (pathname);
+
   state.curdepth = 0;
   state.starting_path_length = strlen (pathname);
 
-  if (0 == strcmp(pathname, parent_dir)
+  if (0 == *base
       || 0 == strcmp(parent_dir, "."))
     {
       dirchange = 0;
@@ -996,7 +989,7 @@ at_top (char *pathname,
 	    error (0, errno, "%s",
 		   safely_quote_err_filename(0, what));
 	  else
-	    error (0, 0, "Failed to safely change directory into %s",
+	    error (0, 0, _("Failed to safely change directory into %s"),
 		   safely_quote_err_filename(0, parent_dir));
 	    
 	  /* We can't process this command-line argument. */
@@ -1072,7 +1065,7 @@ static int dir_curr = -1;
 
 
 
-/* We've detected a filesystem loop.   This is caused by one of 
+/* We've detected a file system loop.   This is caused by one of 
  * two things:
  *
  * 1. Option -L is in effect and we've hit a symbolic link that 
@@ -1112,8 +1105,8 @@ issue_loop_warning(const char *name, const char *pathname, int level)
 	    safely_quote_err_filename(0, pathname),
 	    distance,
 	    (distance == 1 ?
-	     _("level higher in the filesystem hierarchy") :
-	     _("levels higher in the filesystem hierarchy")));
+	     _("level higher in the file system hierarchy") :
+	     _("levels higher in the file system hierarchy")));
     }
 }
 
@@ -1282,7 +1275,7 @@ process_dir (char *pathname, char *name, int pathlen, const struct stat *statp,
   
   if (dirinfo == NULL)
     {
-      assert(errno != 0);
+      assert (errno != 0);
       error (0, errno, "%s", safely_quote_err_filename(0, pathname));
       state.exit_status = 1;
     }
@@ -1397,12 +1390,12 @@ process_dir (char *pathname, char *name, int pathlen, const struct stat *statp,
 		  /* This is a subdirectory, but the number of directories we 
 		   * have found now exceeds the number we would expect given 
 		   * the hard link count on the parent.   This is likely to be 
-		   * a bug in the filesystem driver (e.g. Linux's 
-		   * /proc filesystem) or may just be a fact that the OS 
+		   * a bug in the file system driver (e.g. Linux's 
+		   * /proc file system) or may just be a fact that the OS 
 		   * doesn't really handle hard links with Unix semantics.
 		   * In the latter case, -noleaf should be used routinely.
 		   */
-		  error(0, 0, _("WARNING: Hard link count is wrong for %s (saw only st_nlink=%d but we already saw %d subdirectories): this may be a bug in your filesystem driver.  Automatically turning on find's -noleaf option.  Earlier results may have failed to include directories that should have been searched."),
+		  error(0, 0, _("WARNING: Hard link count is wrong for %s (saw only st_nlink=%d but we already saw %d subdirectories): this may be a bug in your file system driver.  Automatically turning on find's -noleaf option.  Earlier results may have failed to include directories that should have been searched."),
 			safely_quote_err_filename(0, pathname),
 			statp->st_nlink,
 			dircount);
@@ -1414,7 +1407,7 @@ process_dir (char *pathname, char *name, int pathlen, const struct stat *statp,
 		}
 	      
 	      /* Normal case optimization.  On normal Unix
-		 filesystems, a directory that has no subdirectories
+		 file systems, a directory that has no subdirectories
 		 has two links: its name, and ".".  Any additional
 		 links are to the ".." entries of its subdirectories.
 		 Once we have processed as many subdirectories as
@@ -1432,7 +1425,7 @@ process_dir (char *pathname, char *name, int pathlen, const struct stat *statp,
 	    }
 	  else
 	    {
-	      /* There might be weird (e.g., CD-ROM or MS-DOS) filesystems
+	      /* There might be weird (e.g., CD-ROM or MS-DOS) file systems
 		 mounted, which don't have Unix-like directory link counts. */
 	      process_path (cur_path, cur_name, false, pathname, mode);
 	    }
@@ -1509,6 +1502,6 @@ process_dir (char *pathname, char *name, int pathlen, const struct stat *statp,
       /* Make sure we hasn't used the variable subdirs_left if we knew
        * we shouldn't do so.
        */
-      assert(0 == subdirs_left || options.no_leaf_check);
+      assert (0 == subdirs_left || options.no_leaf_check);
     }
 }
