@@ -2,10 +2,10 @@
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 2000, 2001, 2003, 
                  2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-   USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <config.h>
@@ -43,19 +41,17 @@
 #include "fts_.h"
 #include "getdate.h"
 #include "error.h"
-#include "gnulib-version.h"
+#include "findutils-version.h"
 
-#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
-#else
-#include <sys/file.h>
-#endif
+
 
 /* The presence of unistd.h is assumed by gnulib these days, so we 
  * might as well assume it too. 
  */
 /* We need <unistd.h> for isatty(). */
 #include <unistd.h> 
+#include <sys/stat.h>
 
 #if ENABLE_NLS
 # include <libintl.h>
@@ -164,25 +160,44 @@ static boolean parse_quit          PARAMS((const struct parser_table*, char *arg
 boolean parse_print             PARAMS((const struct parser_table*, char *argv[], int *arg_ptr));
 
 
-static boolean insert_type PARAMS((char **argv, int *arg_ptr, const struct parser_table *entry, PRED_FUNC which_pred));
-static boolean insert_regex PARAMS((char *argv[], int *arg_ptr, const struct parser_table *entry, int regex_options));
+static boolean insert_type PARAMS((char **argv, int *arg_ptr,
+				   const struct parser_table *entry,
+				   PRED_FUNC which_pred));
+static boolean insert_regex PARAMS((char *argv[], int *arg_ptr,
+				    const struct parser_table *entry,
+				    int regex_options));
 static boolean insert_fprintf (struct format_val *vec,
 			       const struct parser_table *entry,
 			       PRED_FUNC func,
 			       const char *format);
 
-static struct segment **make_segment PARAMS((struct segment **segment, char *format, int len,
-					     int kind, char format_char, char aux_format_char,
+static struct segment **make_segment PARAMS((struct segment **segment,
+					     char *format, int len,
+					     int kind, char format_char,
+					     char aux_format_char,
 					     struct predicate *pred));
-static boolean insert_exec_ok PARAMS((const char *action, const struct parser_table *entry, int dirfd, char *argv[], int *arg_ptr));
-static boolean get_comp_type PARAMS((const char **str, enum comparison_type *comp_type));
-static boolean get_relative_timestamp PARAMS((const char *str, struct time_val *tval, time_t origin, double sec_per_unit, const char *overflowmessage));
-static boolean get_num PARAMS((const char *str, uintmax_t *num, enum comparison_type *comp_type));
-static struct predicate* insert_num PARAMS((char *argv[], int *arg_ptr, const struct parser_table *entry));
+static boolean insert_exec_ok PARAMS((const char *action,
+				      const struct parser_table *entry,
+				      int dirfd,
+				      char *argv[],
+				      int *arg_ptr));
+static boolean get_comp_type PARAMS((const char **str,
+				     enum comparison_type *comp_type));
+static boolean get_relative_timestamp PARAMS((const char *str,
+					      struct time_val *tval,
+					      time_t origin,
+					      double sec_per_unit,
+					      const char *overflowmessage));
+static boolean get_num PARAMS((const char *str,
+			       uintmax_t *num,
+			       enum comparison_type *comp_type));
+static struct predicate* insert_num PARAMS((char *argv[], int *arg_ptr,
+					    const struct parser_table *entry));
 static void open_output_file (const char *path, struct format_val *p);
 static void open_stdout (struct format_val *p);
 static boolean stream_is_tty(FILE *fp);
-static boolean parse_noop PARAMS((const struct parser_table* entry, char **argv, int *arg_ptr));
+static boolean parse_noop PARAMS((const struct parser_table* entry,
+				  char **argv, int *arg_ptr));
 
 #define PASTE(x,y) x##y
 #define STRINGIFY(s) #s
@@ -283,7 +298,7 @@ static struct parser_table const parse_table[] =
   PARSE_PUNCTUATION("or",                    or),	     /* GNU */
   PARSE_ACTION     ("ok",                    ok),
   PARSE_ACTION     ("okdir",                 okdir), /* GNU (-execdir is BSD) */
-  PARSE_TEST       ("path",                  path), /* GNU, HP-UX, GNU prefers wholename */
+  PARSE_TEST       ("path",                  path), /* GNU, HP-UX, RMS prefers wholename, but anyway soon POSIX */
   PARSE_TEST       ("perm",                  perm),
   PARSE_ACTION     ("print",                 print),
   PARSE_ACTION     ("print0",                print0),	     /* GNU */
@@ -303,7 +318,7 @@ static struct parser_table const parse_table[] =
   PARSE_TEST       ("used",                  used),	     /* GNU */
   PARSE_TEST       ("user",                  user),
   PARSE_OPTION     ("warn",                  warn),	     /* GNU */
-  PARSE_TEST_NP    ("wholename",             wholename), /* GNU, replaces -path */
+  PARSE_TEST_NP    ("wholename",             wholename), /* GNU, replaced -path, but anyway -path will soon be in POSIX */
   {ARG_TEST,       "writable",               parse_accesscheck, pred_writable}, /* GNU, 4.3.0+ */
   PARSE_OPTION     ("xdev",                  xdev),
   PARSE_TEST       ("xtype",                 xtype),	     /* GNU */
@@ -372,8 +387,7 @@ get_stat_Ytime(const struct stat *p,
       *ret = get_stat_mtime(p);
       return 1;
     default:
-      assert(0);
-      abort();
+      assert (0);
       abort();
     }
 }
@@ -414,7 +428,9 @@ set_follow_state(enum SymlinkOption opt)
 
 
 void
-parse_begin_user_args (char **args, int argno, const struct predicate *last, const struct predicate *predicates)
+parse_begin_user_args (char **args, int argno,
+		       const struct predicate *last,
+		       const struct predicate *predicates)
 {
   (void) args;
   (void) argno;
@@ -424,7 +440,9 @@ parse_begin_user_args (char **args, int argno, const struct predicate *last, con
 }
 
 void 
-parse_end_user_args (char **args, int argno, const struct predicate *last, const struct predicate *predicates)
+parse_end_user_args (char **args, int argno,
+		     const struct predicate *last,
+		     const struct predicate *predicates)
 {
   /* does nothing */
   (void) args;
@@ -751,7 +769,9 @@ parse_d (const struct parser_table* entry, char **argv, int *arg_ptr)
   if (options.warnings)
     {
       error (0, 0,
-	     _("warning: the -d option is deprecated; please use -depth instead, because the latter is a POSIX-compliant feature."));
+	     _("warning: the -d option is deprecated; please use "
+	       "-depth instead, because the latter is a "
+	       "POSIX-compliant feature."));
     }
   return parse_depth(entry, argv, arg_ptr);
 }
@@ -876,7 +896,7 @@ static float estimate_fstype_success_rate(const char *fsname)
   if (0 == stat(dir, &dir_stat))
     {
       const char *fstype = filesystem_type(&dir_stat, dir);
-      /* Assume most files are on the same filesystem type as the root fs. */
+      /* Assume most files are on the same file system type as the root fs. */
       if (0 == strcmp(fsname, fstype))
 	  return 0.7f;
       else
@@ -1129,12 +1149,18 @@ fnmatch_sanitycheck(void)
 }
 
 
-static boolean 
+static boolean
 check_name_arg(const char *pred, const char *arg)
 {
-  if (strchr(arg, '/'))
+  if (options.warnings && strchr(arg, '/'))
     {
-      error(0, 0,_("warning: Unix filenames usually don't contain slashes (though pathnames do).  That means that '%s %s' will probably evaluate to false all the time on this system.  You might find the '-wholename' test more useful, or perhaps '-samefile'.  Alternatively, if you are using GNU grep, you could use 'find ... -print0 | grep -FzZ %s'."),
+      error(0, 0,_("warning: Unix filenames usually don't contain slashes "
+		   "(though pathnames do).  That means that '%s %s' will "
+		   "probably evaluate to false all the time on this system.  "
+		   "You might find the '-wholename' test more useful, or "
+		   "perhaps '-samefile'.  Alternatively, if you are using "
+		   "GNU grep, you could "
+		   "use 'find ... -print0 | grep -FzZ %s'."),
 	    pred,
 	    safely_quote_err_filename(0, arg),
 	    safely_quote_err_filename(1, arg));
@@ -1189,27 +1215,24 @@ parse_inum (const struct parser_table* entry, char **argv, int *arg_ptr)
 static boolean
 parse_ipath (const struct parser_table* entry, char **argv, int *arg_ptr)
 {
-  error (0, 0,
-	 _("warning: the predicate -ipath is deprecated; please use -iwholename instead."));
-  
-  return parse_iwholename(entry, argv, arg_ptr);
+  const char *name;
+
+  fnmatch_sanitycheck ();
+  if (collect_arg (argv, arg_ptr, &name))
+    {
+      struct predicate *our_pred = insert_primary_withpred (entry, pred_ipath);
+      our_pred->need_stat = our_pred->need_type = false;
+      our_pred->args.str = name;
+      our_pred->est_success_rate = estimate_pattern_match_rate (name, 0);
+      return true;
+    }
+  return false;
 }
 
 static boolean
 parse_iwholename (const struct parser_table* entry, char **argv, int *arg_ptr)
 {
-  const char *name;
-
-  fnmatch_sanitycheck();
-  if (collect_arg(argv, arg_ptr, &name))
-    {
-      struct predicate *our_pred = insert_primary_withpred (entry, pred_ipath);
-      our_pred->need_stat = our_pred->need_type = false;
-      our_pred->args.str = name;
-      our_pred->est_success_rate = estimate_pattern_match_rate(name, 0);
-      return true;
-    }
-  return false;
+  return parse_ipath (entry, argv, arg_ptr);
 }
 
 static boolean
@@ -1303,7 +1326,10 @@ parse_mindepth (const struct parser_table* entry, char **argv, int *arg_ptr)
 
 
 static boolean
-do_parse_xmin (const struct parser_table* entry, char **argv, int *arg_ptr, enum xval xv)
+do_parse_xmin (const struct parser_table* entry,
+	       char **argv,
+	       int *arg_ptr,
+	       enum xval xv)
 {
   const char *minutes;
 
@@ -1417,7 +1443,7 @@ parse_newerXY (const struct parser_table* entry, char **argv, int *arg_ptr)
       char x, y;
       const char validchars[] = "aBcmt";
       
-      assert(0 == strncmp("-newer", argv[*arg_ptr], 6));
+      assert (0 == strncmp("-newer", argv[*arg_ptr], 6));
       x = argv[*arg_ptr][6];
       y = argv[*arg_ptr][7];
 
@@ -1465,8 +1491,8 @@ parse_newerXY (const struct parser_table* entry, char **argv, int *arg_ptr)
 	      our_pred->args.reftime.xval = XVAL_MTIME;
 	      break;
 	    default:
-	      assert(strchr(validchars, x));
-	      assert(0);
+	      assert (strchr(validchars, x));
+	      assert (0);
 	    }
 	  
 	  if ('t' == y)
@@ -1500,9 +1526,9 @@ parse_newerXY (const struct parser_table* entry, char **argv, int *arg_ptr)
 	  our_pred->est_success_rate = estimate_timestamp_success_rate(our_pred->args.reftime.ts.tv_sec);
 	  (*arg_ptr)++;
 	  
-	  assert(our_pred->pred_func != NULL);
-	  assert(our_pred->pred_func == pred_newerXY);
-	  assert(our_pred->need_stat);
+	  assert (our_pred->pred_func != NULL);
+	  assert (our_pred->pred_func == pred_newerXY);
+	  assert (our_pred->need_stat);
 	  return true;
 	}
     }
@@ -1657,22 +1683,18 @@ parse_or (const struct parser_table* entry, char **argv, int *arg_ptr)
   return true;
 }
 
-/* -path is deprecated (at RMS's request) in favour of 
- * -iwholename.   See the node "GNU Manuals" in standards.texi
- * for the rationale for this (basically, GNU prefers the use 
- * of the phrase "file name" to "path name".
+/* For some time, -path was deprecated (at RMS's request) in favour of
+ * -iwholename.  See the node "GNU Manuals" in standards.texi for the
+ * rationale for this (basically, GNU prefers the use of the phrase
+ * "file name" to "path name".
  *
  * We do not issue a warning that this usage is deprecated
- * since HPUX find supports this predicate also.
+ * since
+ * (a) HPUX find supports this predicate also and 
+ * (b) it will soon be in POSIX anyway.
  */
 static boolean
 parse_path (const struct parser_table* entry, char **argv, int *arg_ptr)
-{
-  return parse_wholename(entry, argv, arg_ptr);
-}
-
-static boolean
-parse_wholename (const struct parser_table* entry, char **argv, int *arg_ptr)
 {
   const char *name;
   if (collect_arg(argv, arg_ptr, &name))
@@ -1680,12 +1702,18 @@ parse_wholename (const struct parser_table* entry, char **argv, int *arg_ptr)
       struct predicate *our_pred = insert_primary_withpred (entry, pred_path);
       our_pred->need_stat = our_pred->need_type = false;
       our_pred->args.str = name;
-      our_pred->est_success_rate = estimate_pattern_match_rate(name, 0);
+      our_pred->est_success_rate = estimate_pattern_match_rate (name, 0);
       return true;
     }
   return false;
 }
 
+static boolean
+parse_wholename (const struct parser_table* entry, char **argv, int *arg_ptr)
+{
+  return parse_path (entry, argv, arg_ptr);
+}
+
 static boolean
 parse_perm (const struct parser_table* entry, char **argv, int *arg_ptr)
 {
@@ -1923,7 +1951,10 @@ parse_regex (const struct parser_table* entry, char **argv, int *arg_ptr)
 }
 
 static boolean
-insert_regex (char **argv, int *arg_ptr, const struct parser_table *entry, int regex_options)
+insert_regex (char **argv,
+	      int *arg_ptr,
+	      const struct parser_table *entry,
+	      int regex_options)
 {
   const char *rx;
   if (collect_arg(argv, arg_ptr, &rx))
@@ -1932,11 +1963,10 @@ insert_regex (char **argv, int *arg_ptr, const struct parser_table *entry, int r
       const char *error_message;
       struct predicate *our_pred = insert_primary_withpred (entry, pred_regex);
       our_pred->need_stat = our_pred->need_type = false;
-      re = (struct re_pattern_buffer *)
-	xmalloc (sizeof (struct re_pattern_buffer));
+      re = xmalloc (sizeof (struct re_pattern_buffer));
       our_pred->args.regex = re;
       re->allocated = 100;
-      re->buffer = (unsigned char *) xmalloc (re->allocated);
+      re->buffer = xmalloc (re->allocated);
       re->fastmap = NULL;
       
       re_set_syntax(regex_options);
@@ -2024,7 +2054,9 @@ parse_size (const struct parser_table* entry, char **argv, int *arg_ptr)
   /* TODO: accept fractional megabytes etc. ? */
   if (!get_num (argv[*arg_ptr], &num, &c_type))
     {
-      error(1, 0, _("Invalid argument `%s%c' to -size"), argv[*arg_ptr], (int)suffix);
+      error(1, 0,
+	    _("Invalid argument `%s%c' to -size"),
+	    argv[*arg_ptr], (int)suffix);
       return false;
     }
   our_pred = insert_primary (entry);
@@ -2073,7 +2105,7 @@ parse_samefile (const struct parser_table* entry, char **argv, int *arg_ptr)
     {
       if (options.open_nofollow_available)
 	{
-	  assert(O_NOFOLLOW != 0);
+	  assert (O_NOFOLLOW != 0);
 	  openflags |= O_NOFOLLOW;
 	  fd = -1;		/* safe to open it. */
 	}
@@ -2103,7 +2135,7 @@ parse_samefile (const struct parser_table* entry, char **argv, int *arg_ptr)
       fd = -1;			/* safe to open it without O_NOFOLLOW */
     }
 
-  assert(fd != -3);		/* check we made a decision */
+  assert (fd != -3);		/* check we made a decision */
   if (fd == -1)
     {
       /* Race condition here.  The file might become a
@@ -2197,10 +2229,13 @@ parse_samefile (const struct parser_table* entry, char **argv, int *arg_ptr)
  * uneven. 
  */
 static boolean
-parse_show_control_chars (const struct parser_table* entry, char **argv, int *arg_ptr)
+parse_show_control_chars (const struct parser_table* entry,
+			  char **argv,
+			  int *arg_ptr)
 {
   const char *arg;
-  const char *errmsg = _("The -show-control-chars option takes a single argument which "
+  const char *errmsg = _("The -show-control-chars option takes "
+			 "a single argument which "
 			 "must be 'literal' or 'safe'");
   
   if ((argv == NULL) || (argv[*arg_ptr] == NULL))
@@ -2354,7 +2389,6 @@ parse_user (const struct parser_table* entry, char **argv, int *arg_ptr)
 static boolean
 parse_version (const struct parser_table* entry, char **argv, int *arg_ptr)
 {
-  extern char *version_string;
   int features = 0;
   int flags;
   
@@ -2362,9 +2396,7 @@ parse_version (const struct parser_table* entry, char **argv, int *arg_ptr)
   (void) arg_ptr;
   (void) entry;
   
-  fflush (stderr);
-  printf (_("GNU find version %s\n"), version_string);
-  printf (_("Built using GNU gnulib version %s\n"), gnulib_version);
+  display_findutils_version("find");
   printf (_("Features enabled: "));
   
 #if CACHE_IDS
@@ -2379,16 +2411,16 @@ parse_version (const struct parser_table* entry, char **argv, int *arg_ptr)
   printf("DEBUG_STAT ");
   ++features;
 #endif
-#if defined(USE_STRUCT_DIRENT_D_TYPE) && defined(HAVE_STRUCT_DIRENT_D_TYPE)
+#if defined USE_STRUCT_DIRENT_D_TYPE && defined HAVE_STRUCT_DIRENT_D_TYPE
   printf("D_TYPE ");
   ++features;
 #endif
-#if defined(O_NOFOLLOW)
+#if defined O_NOFOLLOW
   printf("O_NOFOLLOW(%s) ",
 	 (options.open_nofollow_available ? "enabled" : "disabled"));
   ++features;
 #endif
-#if defined(LEAF_OPTIMISATION)
+#if defined LEAF_OPTIMISATION
   printf("LEAF_OPTIMISATION ");
   ++features;
 #endif
@@ -2461,7 +2493,9 @@ parse_xtype (const struct parser_table* entry, char **argv, int *arg_ptr)
 }
 
 static boolean
-insert_type (char **argv, int *arg_ptr, const struct parser_table *entry, PRED_FUNC which_pred)
+insert_type (char **argv, int *arg_ptr,
+	     const struct parser_table *entry,
+	     PRED_FUNC which_pred)
 {
   mode_t type_cell;
   struct predicate *our_pred;
@@ -2728,7 +2762,7 @@ make_segment (struct segment **segment,
   enum EvaluationCost mycost = NeedsNothing;
   char *fmt;
 
-  *segment = (struct segment *) xmalloc (sizeof (struct segment));
+  *segment = xmalloc (sizeof (struct segment));
 
   (*segment)->segkind = kind;
   (*segment)->format_char[0] = format_char;
@@ -2744,8 +2778,8 @@ make_segment (struct segment **segment,
     {
     case KIND_PLAIN:		/* Plain text string, no % conversion. */
     case KIND_STOP:		/* Terminate argument, no newline. */
-      assert(0 == format_char);
-      assert(0 == aux_format_char);
+      assert (0 == format_char);
+      assert (0 == aux_format_char);
       *fmt = '\0';
       if (mycost > pred->p_cost)
 	pred->p_cost = NeedsNothing;
@@ -2753,7 +2787,7 @@ make_segment (struct segment **segment,
       break;
     }
 
-  assert(kind == KIND_FORMAT);
+  assert (kind == KIND_FORMAT);
   switch (format_char)
     {
     case 'l':			/* object of symlink */
@@ -2773,7 +2807,7 @@ make_segment (struct segment **segment,
     case 'B':			/* birth time in user-specified strftime format */
     case 'c':			/* ctime in `ctime' format */
     case 'C':			/* ctime in user-specified strftime format */
-    case 'F':			/* filesystem type */
+    case 'F':			/* file system type */
     case 'g':			/* group name */
     case 'i':			/* inode number */
     case 'M':			/* mode in `ls -l' format (eg., "drwxr-xr-x") */
@@ -2854,8 +2888,17 @@ make_segment (struct segment **segment,
 static void 
 check_path_safety(const char *action, char **argv)
 {
-  const char *path = getenv("PATH");
   char *s;
+  const char *path = getenv("PATH");
+  if (NULL == path)
+    {
+      /* $PATH is not set.  Assume the OS default is safe.
+       * That may not be true on Windows, but I'm not aware 
+       * of a way to get Windows to avoid searching the 
+       * current directory anyway.
+       */
+      return;
+    }
 
   (void)argv;
   
@@ -2864,7 +2907,12 @@ check_path_safety(const char *action, char **argv)
     {
       if (0 == strcmp(s, "."))
 	{
-	  error(1, 0, _("The current directory is included in the PATH environment variable, which is insecure in combination with the %s action of find.  Please remove the current directory from your $PATH (that is, remove \".\" or leading or trailing colons)"),
+	  error(1, 0, _("The current directory is included in the PATH "
+			"environment variable, which is insecure in "
+			"combination with the %s action of find.  "
+			"Please remove the current directory from your "
+			"$PATH (that is, remove \".\" or leading or trailing "
+			"colons)"),
 		action);
 	}
       else if ('/' != s[0])
@@ -2968,12 +3016,14 @@ new_insert_exec_ok (const char *action,
 	       * occur even in the utility name.  This is insecure
 	       * since it means we will be executing a command whose
 	       * name is chosen according to whatever find finds in
-	       * the filesystem.  That can be influenced by an
+	       * the file system.  That can be influenced by an
 	       * attacker.  Hence for -execdir and -okdir this is not
 	       * allowed.  We can specify this as those options are 
 	       * not defined by POSIX.
 	       */
-	      error(1, 0, _("You may not use {} within the utility name for -execdir and -okdir, because this is a potential security problem."));
+	      error(1, 0, _("You may not use {} within the utility name for "
+			    "-execdir and -okdir, because this is a potential "
+			    "security problem."));
 	    }
 	}
     }
@@ -3085,7 +3135,11 @@ new_insert_exec_ok (const char *action,
 
 
 static boolean
-insert_exec_ok (const char *action, const struct parser_table *entry, int dirfd, char **argv, int *arg_ptr)
+insert_exec_ok (const char *action,
+		const struct parser_table *entry,
+		int dirfd,
+		char **argv,
+		int *arg_ptr)
 {
   return new_insert_exec_ok(action, entry, dirfd, argv, arg_ptr);
 }
@@ -3177,7 +3231,8 @@ parse_time (const struct parser_table* entry, char *argv[], int *arg_ptr)
   struct time_val tval;
   enum comparison_type comp;
   const char *timearg, *orig_timearg;
-  const char *errmsg = "arithmetic overflow while converting %s days to a number of seconds";
+  const char *errmsg = "arithmetic overflow while converting %s "
+    "days to a number of seconds";
   time_t origin;
 
   if (!collect_arg(argv, arg_ptr, &timearg))
@@ -3192,7 +3247,7 @@ parse_time (const struct parser_table* entry, char *argv[], int *arg_ptr)
       /* Remember, we invert the sense of the comparison, so this tests 
        * against COMP_LT instead of COMP_GT... 
        */
-      if (COMP_LT == tval.kind)      
+      if (COMP_LT == comp)      
 	{
 	  uintmax_t expected = origin + (DAYSECS-1);
 	  origin += (DAYSECS-1);
@@ -3228,7 +3283,9 @@ parse_time (const struct parser_table* entry, char *argv[], int *arg_ptr)
 	       (tval.kind == COMP_GT) ? " >" :
 	       ((tval.kind == COMP_LT) ? " <" : ((tval.kind == COMP_EQ) ? ">=" : " ?")));
       t = our_pred->args.reftime.ts.tv_sec;
-      fprintf (stderr, "%ju %s", (uintmax_t) our_pred->args.reftime.ts.tv_sec, ctime (&t));
+      fprintf (stderr, "%ju %s",
+	       (uintmax_t) our_pred->args.reftime.ts.tv_sec,
+	       ctime (&t));
       if (tval.kind == COMP_EQ)
 	{
 	  t = our_pred->args.reftime.ts.tv_sec + DAYSECS;
@@ -3379,4 +3436,3 @@ open_stdout (struct format_val *p)
 {
   open_output_file("/dev/stdout", p);
 }
-
