@@ -2,20 +2,18 @@
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 2000, 2003, 
                  2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-   USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <config.h>
@@ -47,6 +45,7 @@
 #include "stat-time.h"
 #include "dircallback.h"
 #include "error.h"
+#include "verify.h"
 
 #if ENABLE_NLS
 # include <libintl.h>
@@ -110,7 +109,7 @@
    ST_NBLOCKSIZE: Size of blocks used when calculating ST_NBLOCKS.  */
 #ifndef HAVE_STRUCT_STAT_ST_BLOCKS
 # define ST_BLKSIZE(statbuf) DEV_BSIZE
-# if defined(_POSIX_SOURCE) || !defined(BSIZE) /* fileblocks.c uses BSIZE.  */
+# if defined _POSIX_SOURCE || !defined BSIZE /* fileblocks.c uses BSIZE.  */
 #  define ST_NBLOCKS(statbuf) \
   (S_ISREG ((statbuf).st_mode) \
    || S_ISDIR ((statbuf).st_mode) \
@@ -125,16 +124,16 @@
 /* Some systems, like Sequents, return st_blksize of 0 on pipes. */
 # define ST_BLKSIZE(statbuf) ((statbuf).st_blksize > 0 \
 			       ? (statbuf).st_blksize : DEV_BSIZE)
-# if defined(hpux) || defined(__hpux__) || defined(__hpux)
+# if defined hpux || defined __hpux__ || defined __hpux
 /* HP-UX counts st_blocks in 1024-byte units.
-   This loses when mixing HP-UX and BSD filesystems with NFS.  */
+   This loses when mixing HP-UX and BSD file systems with NFS.  */
 #  define ST_NBLOCKSIZE 1024
 # else /* !hpux */
-#  if defined(_AIX) && defined(_I386)
+#  if defined _AIX && defined _I386
 /* AIX PS/2 counts st_blocks in 4K units.  */
 #   define ST_NBLOCKSIZE (4 * 1024)
 #  else /* not AIX PS/2 */
-#   if defined(_CRAY)
+#   if defined _CRAY
 #    define ST_NBLOCKS(statbuf) \
   (S_ISREG ((statbuf).st_mode) \
    || S_ISDIR ((statbuf).st_mode) \
@@ -293,6 +292,8 @@ pred_timewindow(struct timespec ts, struct predicate const *pred_ptr, int window
 	return (delta >= 0.0 && delta < window);
       }
     }
+  assert (0);
+  abort ();
 }
 
 
@@ -319,7 +320,7 @@ boolean
 pred_anewer (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   (void) &pathname;
-  assert(COMP_GT == pred_ptr->args.reftime.kind);
+  assert (COMP_GT == pred_ptr->args.reftime.kind);
   return compare_ts(get_stat_atime(stat_buf), pred_ptr->args.reftime.ts) > 0;
 }
 
@@ -352,7 +353,7 @@ pred_cnewer (const char *pathname, struct stat *stat_buf, struct predicate *pred
 {
   (void) pathname;
   
-  assert(COMP_GT == pred_ptr->args.reftime.kind);
+  assert (COMP_GT == pred_ptr->args.reftime.kind);
   return compare_ts(get_stat_ctime(stat_buf), pred_ptr->args.reftime.ts) > 0;
 }
 
@@ -407,7 +408,7 @@ pred_delete (const char *pathname, struct stat *stat_buf, struct predicate *pred
 		}
 	    }
 	}
-      error (0, errno, "cannot delete %s",
+      error (0, errno, _("cannot delete %s"),
 	     safely_quote_err_filename(0, pathname));
       return false;
     }
@@ -627,7 +628,7 @@ mode_to_filetype(mode_t m)
 static double 
 file_sparseness(const struct stat *p)
 {
-#if defined(HAVE_STRUCT_STAT_ST_BLOCKS)
+#if defined HAVE_STRUCT_STAT_ST_BLOCKS
   if (0 == p->st_size)
     {
       if (0 == p->st_blocks)
@@ -740,9 +741,13 @@ do_fprintf(struct format_val *dest,
 	  break;
 	case 'f':		/* base name of path */
 	  /* sanitised */
-	  checked_print_quoted (dest, segment->text, base_name (pathname));
+	  {
+	    char *base = base_name (pathname);
+	    checked_print_quoted (dest, segment->text, base);
+	    free (base);
+	  }
 	  break;
-	case 'F':		/* filesystem type */
+	case 'F':		/* file system type */
 	  /* trusted */
 	  checked_print_quoted (dest, segment->text, filesystem_type (stat_buf, pathname));
 	  break;
@@ -969,7 +974,7 @@ do_fprintf(struct format_val *dest,
 					   human_ceiling, 1, 1));
 	  break;
 
-	  /* %Y: type of filesystem entry like `ls -l`: 
+	  /* %Y: type of file system entry like `ls -l`: 
 	   *     (d,-,l,s,p,b,c,n) n=nonexistent(symlink) 
 	   */
 	case 'Y':		/* in case of symlink */
@@ -1065,7 +1070,7 @@ pred_fprintf (const char *pathname, struct stat *stat_buf, struct predicate *pre
 	      valid = 1;
 	      break;
 	    default:
-	      assert(0);
+	      assert (0);
 	      abort ();
 	    }
 	  /* We trust the output of format_date not to contain 
@@ -1149,20 +1154,38 @@ pred_ilname (const char *pathname, struct stat *stat_buf, struct predicate *pred
   return match_lname (pathname, stat_buf, pred_ptr, true);
 }
 
-boolean
-pred_iname (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
+/* Common code between -name, -iname.  PATHNAME is being visited, STR
+   is name to compare basename against, and FLAGS are passed to
+   fnmatch.  */
+static boolean
+pred_name_common (const char *pathname, const char *str, int flags)
 {
-  const char *base;
-
-  (void) stat_buf;
+  /* Prefer last_component over base_name, to avoid malloc when
+     possible.  */
+  char *base = last_component (pathname);
 
+  /* base is empty only if pathname is a file system root.  But recall
+     that 'find / -name /' is one of the few times where a '/' in the
+     -name must actually find something.  */
+  if (!*base)
+    {
+      boolean b;
+      base = base_name (pathname);
+      b = fnmatch (str, base, flags) == 0;
+      free (base);
+      return b;
+    }
   /* FNM_PERIOD is not used here because POSIX requires that it not be.
    * See http://standards.ieee.org/reading/ieee/interp/1003-2-92_int/pasc-1003.2-126.html
    */
-  base = base_name (pathname);
-  if (fnmatch (pred_ptr->args.str, base, FNM_CASEFOLD) == 0)
-    return (true);
-  return (false);
+  return fnmatch (str, base, flags) == 0;
+}
+
+boolean
+pred_iname (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
+{
+  (void) stat_buf;
+  return pred_name_common (pathname, pred_ptr->args.str, FNM_CASEFOLD);
 }
 
 boolean
@@ -1270,17 +1293,8 @@ pred_mtime (const char *pathname, struct stat *stat_buf, struct predicate *pred_
 boolean
 pred_name (const char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
-  const char *base;
-
   (void) stat_buf;
-  base = base_name (pathname);
-
-  /* FNM_PERIOD is not used here because POSIX requires that it not be.
-   * See http://standards.ieee.org/reading/ieee/interp/1003-2-92_int/pasc-1003.2-126.html
-   */
-  if (fnmatch (pred_ptr->args.str, base, 0) == 0)
-    return (true);
-  return (false);
+  return pred_name_common (pathname, pred_ptr->args.str, 0);
 }
 
 boolean
@@ -1294,7 +1308,7 @@ pred_newer (const char *pathname, struct stat *stat_buf, struct predicate *pred_
 {
   (void) pathname;
   
-  assert(COMP_GT == pred_ptr->args.reftime.kind);
+  assert (COMP_GT == pred_ptr->args.reftime.kind);
   return compare_ts(get_stat_mtime(stat_buf), pred_ptr->args.reftime.ts) > 0;
 }
 
@@ -1304,12 +1318,12 @@ pred_newerXY (const char *pathname, struct stat *stat_buf, struct predicate *pre
   struct timespec ts;
   boolean collected = false;
   
-  assert(COMP_GT == pred_ptr->args.reftime.kind);
+  assert (COMP_GT == pred_ptr->args.reftime.kind);
   
   switch (pred_ptr->args.reftime.xval)
     {
     case XVAL_TIME:
-      assert(pred_ptr->args.reftime.xval != XVAL_TIME);
+      assert (pred_ptr->args.reftime.xval != XVAL_TIME);
       return false;
 
     case XVAL_ATIME:
@@ -1340,7 +1354,7 @@ pred_newerXY (const char *pathname, struct stat *stat_buf, struct predicate *pre
       break;
     }
   
-  assert(collected);
+  assert (collected);
   return compare_ts(ts, pred_ptr->args.reftime.ts) > 0;
 }
 
@@ -1667,7 +1681,7 @@ pred_type (const char *pathname, struct stat *stat_buf, struct predicate *pred_p
   mode_t mode;
   mode_t type = pred_ptr->args.type;
 
-  assert(state.have_type);
+  assert (state.have_type);
 
   if (0 == state.type)
     {
@@ -1860,7 +1874,7 @@ prep_child_for_exec (boolean close_stdin, int dirfd)
    */
   if (dirfd != AT_FDCWD)
     {
-      assert(dirfd >= 0);
+      assert (dirfd >= 0);
       if (0 != fchdir(dirfd))
 	{
 	  /* If we cannot execute our command in the correct directory,
@@ -1886,8 +1900,8 @@ launch (const struct buildcmd_control *ctl,
 
   if (!execp->use_current_dir)
     {
-      assert(starting_desc >= 0);
-      assert(execp->dirfd == starting_desc);
+      assert (starting_desc >= 0);
+      assert (execp->dirfd == starting_desc);
     }
   
 	
@@ -1911,7 +1925,7 @@ launch (const struct buildcmd_control *ctl,
   if (child_pid == 0)
     {
       /* We are the child. */
-      assert(starting_desc >= 0);
+      assert (starting_desc >= 0);
       if (!prep_child_for_exec(execp->close_stdin, execp->dirfd))
 	{
 	  _exit(1);
@@ -2025,7 +2039,7 @@ format_date (struct timespec ts, int kind)
    * demonstrating that the performance difference is actually
    * measurable.
    */
-  assert(sizeof(buf) >= LONGEST_HUMAN_READABLE);
+  verify (sizeof(buf) >= LONGEST_HUMAN_READABLE);
 
   charsprinted = 0;
   need_ns_suffix = 0;
@@ -2067,7 +2081,7 @@ format_date (struct timespec ts, int kind)
        */
       ns_buf[0] = 0;
       charsprinted = snprintf(ns_buf, NS_BUF_LEN, ".%09ld0", (long int)ts.tv_nsec);
-      assert(charsprinted < NS_BUF_LEN);
+      assert (charsprinted < NS_BUF_LEN);
     }
 
   if (kind != '@'
@@ -2079,7 +2093,7 @@ format_date (struct timespec ts, int kind)
        */
       if (need_ns_suffix)
 	{
-	  assert((sizeof buf - strlen(buf)) > strlen(ns_buf));
+	  assert ((sizeof buf - strlen(buf)) > strlen(ns_buf));
 	  strcat(buf, ns_buf);
 	}
       return buf;
@@ -2094,8 +2108,8 @@ format_date (struct timespec ts, int kind)
        */
       char *p = human_readable (ts.tv_sec < 0 ? -w : w, buf + 1,
 				human_ceiling, 1, 1);
-      assert(p > buf);
-      assert(p < (buf + (sizeof buf)));
+      assert (p > buf);
+      assert (p < (buf + (sizeof buf)));
       if (ts.tv_sec < 0)
 	*--p = '-'; /* XXX: Ugh, relying on internal details of human_readable(). */
 
@@ -2108,7 +2122,7 @@ format_date (struct timespec ts, int kind)
 	{
 	  len = strlen(p);
 	  used = (p-buf) + len;	/* Offset into buf of current end */
-	  assert(sizeof buf > used); /* Ensure we can perform subtraction safely. */
+	  assert (sizeof buf > used); /* Ensure we can perform subtraction safely. */
 	  remaining = sizeof buf - used - 1u; /* allow space for NUL */
 	  
 	  if (strlen(ns_buf) >= remaining)
@@ -2117,7 +2131,7 @@ format_date (struct timespec ts, int kind)
 		    "charsprinted=%ld but remaining=%lu: ns_buf=%s",
 		    (long)charsprinted, (unsigned long)remaining, ns_buf);
 	    }
-	  assert(strlen(ns_buf) < remaining);
+	  assert (strlen(ns_buf) < remaining);
 	  strcat(p, ns_buf);
 	}
       return p;
@@ -2146,14 +2160,14 @@ ctime_format (struct timespec ts)
   ptm = localtime(&ts.tv_sec);
   if (ptm)
     {
-      assert(ptm->tm_wday >=  0);
-      assert(ptm->tm_wday <   7);
-      assert(ptm->tm_mon  >=  0);
-      assert(ptm->tm_mon  <  12);
-      assert(ptm->tm_hour >=  0);
-      assert(ptm->tm_hour <  24);
-      assert(ptm->tm_min  <  60);
-      assert(ptm->tm_sec  <= 61); /* allows 2 leap seconds. */
+      assert (ptm->tm_wday >=  0);
+      assert (ptm->tm_wday <   7);
+      assert (ptm->tm_mon  >=  0);
+      assert (ptm->tm_mon  <  12);
+      assert (ptm->tm_hour >=  0);
+      assert (ptm->tm_hour <  24);
+      assert (ptm->tm_min  <  60);
+      assert (ptm->tm_sec  <= 61); /* allows 2 leap seconds. */
       
       /* wkday mon mday hh:mm:ss.nnnnnnnnn yyyy */
       nout = snprintf(resultbuf, TIME_BUF_LEN,
@@ -2167,7 +2181,7 @@ ctime_format (struct timespec ts)
 		      (long int)ts.tv_nsec,
 		      1900 + ptm->tm_year);
       
-      assert(nout < TIME_BUF_LEN);
+      assert (nout < TIME_BUF_LEN);
       return resultbuf;
     }
   else
@@ -2294,7 +2308,7 @@ void show_success_rates(const struct predicate *p)
 void
 pred_sanity_check(const struct predicate *predicates)
 {
-  /* Do nothing, since assert() is a no-op with _NDEBUG set */
+  /* Do nothing, since assert is a no-op with _NDEBUG set */
   return;
 }
 #else
@@ -2306,10 +2320,10 @@ pred_sanity_check(const struct predicate *predicates)
   for (p=predicates; p != NULL; p=p->pred_next)
     {
       /* All predicates must do something. */
-      assert(p->pred_func != NULL);
+      assert (p->pred_func != NULL);
 
       /* All predicates must have a parser table entry. */
-      assert(p->parser_entry != NULL);
+      assert (p->parser_entry != NULL);
       
       /* If the parser table tells us that just one predicate function is 
        * possible, verify that that is still the one that is in effect.
@@ -2318,7 +2332,7 @@ pred_sanity_check(const struct predicate *predicates)
        */
       if (p->parser_entry->pred_func)
 	{
-	  assert(p->parser_entry->pred_func == p->pred_func);
+	  assert (p->parser_entry->pred_func == p->pred_func);
 	}
       
       switch (p->parser_entry->type)
@@ -2334,8 +2348,8 @@ pred_sanity_check(const struct predicate *predicates)
 	   */
 	case ARG_OPTION:
 	case ARG_POSITIONAL_OPTION:
-	  assert(p->parser_entry->type != ARG_OPTION);
-	  assert(p->parser_entry->type != ARG_POSITIONAL_OPTION);
+	  assert (p->parser_entry->type != ARG_OPTION);
+	  assert (p->parser_entry->type != ARG_POSITIONAL_OPTION);
 	  break;
 	  
 	case ARG_ACTION:
@@ -2345,7 +2359,7 @@ pred_sanity_check(const struct predicate *predicates)
 	      /* actions other than -prune and -quit should
 	       * inhibit the default -print
 	       */
-	      assert(p->no_default_print);
+	      assert (p->no_default_print);
 	    }
 	  break;
 
@@ -2359,8 +2373,8 @@ pred_sanity_check(const struct predicate *predicates)
 	  /* Punctuation and tests should have no side
 	   * effects and not inhibit default print.
 	   */
-	  assert(!p->no_default_print);
-	  assert(!p->side_effects);
+	  assert (!p->no_default_print);
+	  assert (!p->side_effects);
 	  break;
 	}
     }
