@@ -1,36 +1,33 @@
 /* listfile.c -- run a function in a specific directory
    Copyright (C) 2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-   USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* This file was written by James Youngman, based on gnulib'c at-func.c. 
  */
 
 
-#if HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
+
 
 #include "openat.h"
 #include <stdarg.h>
 #include <stddef.h>
+#include <errno.h>
 
 #include "fcntl--.h"
-#include "lchown.h"
 #include "lstat.h"
 #include "save-cwd.h"
 
@@ -98,4 +95,3 @@ run_in_dir (int dirfd, int (*callback)(void*), void *usercontext)
       return err;
     }
 }
-
